from django.core.management.base import BaseCommand

from domain_event_broker import replay


class Command(BaseCommand):

    help = "Move dead-lettered event back into given subscriber queue"

    def add_arguments(self, parser):
        parser.add_argument('queue', nargs='+', type=str)
        parser.add_argument(
            '--all',
            action='store_true',
            dest='replay_all',
            default=False,
            help='Replay all events in dead letter queue.',
        )
        parser.add_argument(
            '--interactive',
            action='store_true',
            dest='interactive',
            default=False,
            help='Ask for desired action for each event.',
        )

    def interactive_filter(self, body=None, **kwargs):
        self.stdout.write("Please specify action for: '{}'".format(body))
        action = input("(R)eplay, (D)iscard or (L)eave? ")[0].upper()
        return {
            'R': replay.RETRY,
            'D': replay.DISCARD,
            'L': replay.LEAVE,
            }[action]

    def handle(self, *args, **options):
        callback = replay.retry_event
        if options['interactive']:
            callback = self.interactive_filter
        for queue in options['queue']:
            if options['replay_all']:
                remaining = replay.replay_all(queue, callback)
            else:
                remaining = replay.replay_event(queue, callback)
            self.stdout.write("{} dead-lettered events remaining for {}".format(remaining, queue))



from domain_event_broker import publish_domain_event, Subscriber
import pytest
import uuid


class ConsumerError(Exception):
    pass


def raise_error(event):
    raise ConsumerError("Unexpected error")


@pytest.fixture
def dead_letter_message():
    name = 'test-replay'
    subscriber = Subscriber()
    subscriber.register(raise_error, name, ['test.replay'], dead_letter=True)
    data = dict(message=str(uuid.uuid4())[:4])
    publish_domain_event('test.replay', data)
    subscriber.start_consuming(timeout=1.0)
    yield data
    transport = Subscriber()
    transport.channel.queue_delete(queue='test-replay')
    transport.channel.queue_delete(queue='test-replay-dl')



from .transport import Transport
from . import settings


RETRY = 'retry'
LEAVE = 'leave'
DISCARD = 'discard'


def retry_event(**kwargs):
    return RETRY


def replay_event(queue_name, message_callback=retry_event,
                 connection_settings=settings.DEFAULT):
    """
    Move one domain event from a dead-letter queue back into the processing queue.

    :param str queue_name: Name of the queue where to move the event.
    :param function message_callback: A callable that receives the event and
        returns either ``RETRY``, ``LEAVE`` or ``DISCARD``.
    :param str connection_settings:

    :return: The number of messages left in the dead letter queue.
    :rtype: int
    """
    if connection_settings is None:
        return 0
    elif connection_settings is settings.DEFAULT:
        connection_settings = settings.BROKER
    retry_exchange = queue_name + '-retry'
    dead_letter_queue = queue_name + '-dl'
    transport = Transport(connection_settings)
    transport.connect()
    frame, header, body = transport.channel.basic_get(dead_letter_queue)
    if frame is None:
        return 0
    action = message_callback(frame=frame, header=header, body=body)
    if action == RETRY:
        transport.channel.basic_publish(exchange=retry_exchange,
                                        routing_key=frame.routing_key,
                                        body=body,
                                        )
        transport.channel.basic_ack(frame.delivery_tag)
    elif action == DISCARD:
        transport.channel.basic_ack(frame.delivery_tag)
    elif action == LEAVE:
        transport.channel.basic_reject(frame.delivery_tag, requeue=True)
    else:
        transport.channel.basic_reject(frame.delivery_tag, requeue=True)
        raise Exception("Invalid action '{}'".format(action))
    return frame.message_count


def replay_all(queue_name, message_callback=retry_event,
               connection_settings=settings.DEFAULT):
    """
    Replay all messages currently in the dead-letter queue.
    Return number of messages dead-lettered since starting the replay.
    """
    remainder = replay_event(queue_name, message_callback, connection_settings=connection_settings)
    for _ in range(remainder):
        remainder = replay_event(queue_name, message_callback, connection_settings=connection_settings)
    return remainder



SECRET_KEY = 'test'
INSTALLED_APPS = [
    "domain_event_broker.django",
]
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'domain_event_broker',
    }
}



import pika
from domain_event_broker import DomainEvent


def get_queue_size(name, **kwargs):
    connection = pika.BlockingConnection()
    channel = connection.channel()
    q = channel.queue_declare(name, passive=True, **kwargs)
    count = q.method.message_count
    connection.close()
    return count


def check_queue_exists(name, **kwargs):
    connection = pika.BlockingConnection()
    channel = connection.channel()
    try:
        channel.queue_declare(name, passive=True, **kwargs)
    except pika.exceptions.ChannelClosed as error:
        connection.close()
        return error.args[0] != 404
    else:
        connection.close()
        return True


def get_message_from_queue(name, **kwargs):
    connection = pika.BlockingConnection()
    channel = connection.channel()
    method_frame, header, body = channel.basic_get(name)
    if method_frame:
        channel.basic_ack(method_frame.delivery_tag)
        event = DomainEvent.from_json(body)
        return header, event
    else:
        return None, None



#!/usr/bin/env python

import sys
from .test_queues import get_queue

queue_name, binding_key = sys.argv[1:]

queue = get_queue(True, binding_keys=[binding_key], queue_name=queue_name)
queue.receive()



from django.db import transaction
from domain_event_broker import publish_domain_event


def publish_on_commit(*args, **kwargs):
    """
    Send domain event after transaction has been committed to the database. If
    there is no transaction, it'll be sent right away. If atomic blocks are
    nested, it will be sent when exiting the outermost atomic block.

    More information can be found here:

    https://docs.djangoproject.com/en/dev/topics/db/transactions/#performing-actions-after-commit
    """
    def publish():
        publish_domain_event(*args, **kwargs)

    transaction.on_commit(publish)
